<!--
GENERATED FILE - DO NOT EDIT
This file was generated by [MarkdownSnippets](https://github.com/SimonCropp/MarkdownSnippets).
Source File: /doc/mdsource/DisposableObjects.source.md
To change this file edit the source file and then execute ./run_markdown_templates.sh.
-->

<a id="top"></a>

# Disposable Objects

Most of the configuration points in Approval Tests return a "disposable object" (also known as [RAII](/doc/Glossary.md#raii-resource-acquisition-is-initialization).) This means that when that object is deleted (when it goes out of scope), the customisation to the configuration is undone, and the previous configuration is restored.

This means it is important to store the result in a variable, to pay attention to the scope of that variable, so it lines up with how long you wish the configuration to exist.

All of our disposable objects take advantage of the C++17 language feature `[[nodiscard]]`. If you are on C++17, this means that the following line will either give a compiler warning or a compiler error, depending on your setup.

<!-- snippet: disposable_incorrect -->
<a id='snippet-disposable_incorrect'/></a>
```cpp
Approvals::useApprovalsSubdirectory("directory");
```
<sup><a href='/tests/DocTest_Tests/docs/DisposableSamples.cpp#L12-L14' title='File snippet `disposable_incorrect` was extracted from'>snippet source</a> | <a href='#snippet-disposable_incorrect' title='Navigate to start of snippet `disposable_incorrect`'>anchor</a></sup>
<!-- endsnippet -->

If you are on C++14 or below, the compiler will not detect this, but the code is still incorrect.

The code should look like this:

<!-- snippet: disposable_correct -->
<a id='snippet-disposable_correct'/></a>
```cpp
auto disposer = Approvals::useApprovalsSubdirectory("directory");
```
<sup><a href='/tests/DocTest_Tests/docs/DisposableSamples.cpp#L7-L9' title='File snippet `disposable_correct` was extracted from'>snippet source</a> | <a href='#snippet-disposable_correct' title='Navigate to start of snippet `disposable_correct`'>anchor</a></sup>
<!-- endsnippet -->

---

[Back to User Guide](/doc/README.md#top)
